# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('bodystructures', '0002_initial'),
        ('careplans', '0004_initial'),
        ('careteams', '0004_initial'),
        ('conditions', '0004_initial'),
        ('devicedefinitions', '0001_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0002_initial'),
        ('encounters', '0003_initial'),
        ('groups', '0002_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('medicationrequests', '0002_initial'),
        ('medications', '0001_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('provenances', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AsNeeded',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BodySite',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Parameter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureCodes',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureReason',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Reference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceRequestCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceRequestPatientInstruction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instruction_markdown', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OrderDetailParameter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_boolean', models.BooleanField(default=False)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_detail_parameter_code', to='base.codeableconcept')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_detail_parameter_value_codeable_concept', to='base.codeableconcept')),
                ('value_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_detail_parameter_value_period', to='base.period')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_detail_parameter_value_quantity', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_detail_parameter_value_range', to='base.range')),
                ('value_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='order_detail_parameter_value_ratio', to='base.ratio')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OrderDetailParameterFocusReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_device', to='devices.device')),
                ('device_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_device_definition', to='devicedefinitions.devicedefinition')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_identifier', to='base.identifier')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_medication', to='medications.medication')),
                ('medication_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_medication_request', to='medicationrequests.medicationrequest')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OrderDetailParameterFocusCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_codeable_reference_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_codeable_reference_reference', to='servicerequests.orderdetailparameterfocusreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instantiates_uri', models.URLField(null=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('on_hold', 'On Hold'), ('revoked', 'Revoked'), ('completed', 'Completed'), ('entered_in_error', 'Entered in Error'), ('unknown', 'Unknown')], default='active', max_length=255)),
                ('intent', models.CharField(choices=[('proposal', 'Proposal'), ('plan', 'Plan'), ('order', 'Order'), ('original-order', 'Original Order'), ('reflex-order', 'Reflex Order'), ('filler-order', 'Filler Order'), ('instance-order', 'Instance Order'), ('option', 'Option')], default='order', max_length=255)),
                ('priority', models.CharField(choices=[('routine', 'Routine'), ('urgent', 'Urgent'), ('asap', 'ASAP'), ('stat', 'Stat')], default='routine', max_length=255)),
                ('do_not_perform', models.BooleanField(default=False)),
                ('occurrence_date_time', models.DateTimeField(null=True)),
                ('as_needed', models.BooleanField(default=False)),
                ('authored_on', models.DateTimeField(null=True)),
                ('as_needed_for', models.ManyToManyField(blank=True, related_name='service_request_as_needed_for', to='base.codeableconcept')),
                ('body_site', models.ManyToManyField(blank=True, related_name='service_request_body_site', to='base.codeableconcept')),
                ('body_structure', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_body_structure', to='bodystructures.bodystructurereference')),
                ('category', models.ManyToManyField(blank=True, related_name='service_request_category', to='base.codeableconcept')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_encounter', to='encounters.encounterreference')),
                ('focus', models.ManyToManyField(blank=True, related_name='service_request_focus', to='servicerequests.reference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='service_request_identifier', to='base.identifier')),
                ('location', models.ManyToManyField(blank=True, related_name='service_request_location', to='locations.locationcodeablereference')),
                ('note', models.ManyToManyField(blank=True, related_name='service_request_note', to='base.annotation')),
                ('occurrence_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_occurrence_period', to='base.period')),
                ('occurrence_timing', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_occurrence_timing', to='base.timing')),
                ('performer_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.codeableconcept')),
                ('quantity_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_quantity_quantity', to='base.quantity')),
                ('quantity_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_quantity_range', to='base.range')),
                ('quantity_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_quantity_ratio', to='base.ratio')),
                ('relevant_history', models.ManyToManyField(blank=True, related_name='service_request_relevant_history', to='provenances.provenancereference')),
                ('requisition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requisition', to='base.identifier')),
                ('supporting_info', models.ManyToManyField(blank=True, related_name='service_request_supporting_info', to='base.codeablereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceRequestBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_plan', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_based_on_care_paln', to='careplans.careplan')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_based_on_identifier', to='base.identifier')),
                ('medicatoin_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_based_on_medication_request', to='medicationrequests.medicationrequest')),
                ('service_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_based_on_service_request', to='servicerequests.servicerequest')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='service_request_based_on', to='servicerequests.servicerequestbasedonreference'),
        ),
        migrations.CreateModel(
            name='ServiceRequestOrderDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('parameter', models.ManyToManyField(blank=True, related_name='service_request_order_detail_parameter', to='servicerequests.orderdetailparameter')),
                ('parameter_focus', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_order_detail_parameter_focus', to='servicerequests.orderdetailparameterfocuscodeablereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='order_detail',
            field=models.ManyToManyField(blank=True, to='servicerequests.servicerequestorderdetail'),
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='patient_instruction',
            field=models.ManyToManyField(blank=True, related_name='service_request_patient_instruction', to='servicerequests.servicerequestpatientinstruction'),
        ),
        migrations.CreateModel(
            name='ServiceRequestPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_care_team', to='careteams.careteam')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_device', to='devices.device')),
                ('healthcare_service', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_healthcare_service', to='healthcareservices.healthcareservice')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_performer_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='service_request_performer', to='servicerequests.servicerequestperformerreference'),
        ),
        migrations.CreateModel(
            name='ServiceRequestPlanDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_plan_definition_identifier', to='base.identifier')),
                ('service_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_plan_definition_service_request', to='servicerequests.servicerequest')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceRequestPlanDefinitionReferenceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_plan_definition_reference_codeable_reference_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_plan_definition_reference_codeable_reference_reference', to='servicerequests.servicerequestplandefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ServiceRequestReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_codeable_reference_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='service_request_reason', to='servicerequests.servicerequestreasoncodeablereference'),
        ),
        migrations.CreateModel(
            name='ServiceRequestReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('condition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_condition', to='conditions.condition')),
                ('diagnostic_report', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_diagnostic_report', to='diagnosticreports.diagnosticreport')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_identifier', to='base.identifier')),
                ('observation', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_observation', to='observations.observation')),
                ('procedure', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_procedure', to='procedures.procedure')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequestreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_codeable_reference_reference', to='servicerequests.servicerequestreasonreference'),
        ),
        migrations.CreateModel(
            name='ServiceRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reference_identifier', to='base.identifier')),
                ('service_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reference_service_request', to='servicerequests.servicerequest')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='replaces',
            field=models.ManyToManyField(blank=True, related_name='service_request_replaces', to='servicerequests.servicerequestreference'),
        ),
        migrations.CreateModel(
            name='ServiceRequestRequesterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='additional_recipient',
            field=models.ManyToManyField(blank=True, related_name='service_request_additional_recipient', to='servicerequests.servicerequestrequesterreference'),
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='requester',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_requester', to='servicerequests.servicerequestrequesterreference'),
        ),
        migrations.CreateModel(
            name='ServiceRequestSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_subject_device', to='devices.device')),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_subject_group', to='groups.group')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_subject_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_subject_location', to='locations.location')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_subject_patient', to='patients.patient')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_subject', to='servicerequests.servicerequestsubjectreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
        ('devices', '0001_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='signatureonbehalfofreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='signatureonbehalfofreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='signatureonbehalfofreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of_reference_organization', to='organizations.organization'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('encounters', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GenomicStudyAnalysis',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instantiates_uri', models.URLField(null=True)),
                ('title', models.CharField(max_length=200, null=True)),
                ('date', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyAnalysisDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyAnalysisInput',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyAnalysisOutput',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyAnalysisPerformer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyAnalysisPerformerActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyAnalysisProtocolPerformedReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudySubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GenomicStudy',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('registered', 'Registered'), ('available', 'Available'), ('cancelled', 'Cancelled'), ('enter-in-error', 'Enter in Error'), ('unknown', 'Unknown')], max_length=200, null=True)),
                ('start_date', models.DateTimeField(null=True)),
                ('instantiates_uri', models.URLField(null=True)),
                ('description', models.TextField(null=True)),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_encounter', to='encounters.encounterreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='genomic_study_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""molecular sequences urls."""

from django.urls import path

from dfhir.molecularsequences import views

app_name = "molecularsequences"

urlpatterns = [
    path("molecularsequences/", views.MolecularSequenceListView.as_view()),
    path("molecularsequences/<int:pk>/", views.MolecularSequenceDetailView.as_view()),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppointmentBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentEncounterReason',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentParticipant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('required', models.BooleanField(default=False)),
                ('status', models.CharField(choices=[('accepted', 'Accepted'), ('declined', 'Declined'), ('tentative', 'Tentative'), ('needs-action', 'Needs Action')], max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentParticipantActor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceBinaryCommunicationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MonthlyTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('day_of_month', models.PositiveIntegerField()),
                ('month_interval', models.PositiveIntegerField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RecurrenceTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('last_occurrence_date', models.DateField(blank=True, null=True)),
                ('occurrence_count', models.PositiveIntegerField(blank=True, null=True)),
                ('occurrence_date', models.DateField(blank=True, null=True)),
                ('excluding_recurrence_id', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('excluding_date', django.contrib.postgres.fields.ArrayField(base_field=models.DateField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WeeklyTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('monday', models.BooleanField(default=False)),
                ('tuesday', models.BooleanField(default=False)),
                ('wednesday', models.BooleanField(default=False)),
                ('thursday', models.BooleanField(default=False)),
                ('friday', models.BooleanField(default=False)),
                ('saturday', models.BooleanField(default=False)),
                ('sunday', models.BooleanField(default=False)),
                ('week_interval', models.PositiveIntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='YearlyTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('year_interval', models.PositiveIntegerField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Appointment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('proposed', 'Proposed'), ('pending', 'Pending'), ('booked', 'Booked'), ('arrived', 'Arrived'), ('fulfilled', 'Fulfilled'), ('cancelled', 'Cancelled'), ('noshow', 'No Show'), ('entered_in_error', 'Entered in Error'), ('checked_out', 'Checked Out'), ('waitlist', 'Waitlist')], max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('start', models.DateTimeField(null=True)),
                ('end', models.DateTimeField(null=True)),
                ('minutes_duration', models.PositiveIntegerField(null=True)),
                ('cancellation_date', models.DateTimeField(blank=True, help_text='Date of cancellation', null=True)),
                ('recurrence_id', models.PositiveIntegerField(blank=True, null=True)),
                ('occurrence_changed', models.BooleanField(default=False)),
                ('account', models.ManyToManyField(blank=True, related_name='appointment_account', to='accounts.accountreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Contract serializer."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework_recursive.fields import RecursiveField

from dfhir.base.serializers import (
    AnnotationSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    ReferenceSerializer,
    SignatureSerializer,
    SimpleQuantitySerializer,
    TimingSerializer,
)
from dfhir.encounters.serializers import EncounterEpisodeOfCareReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.provenances.serializers import ProvenanceReferenceSerializer

from .models import (
    Contract,
    ContractAuthorReference,
    ContractContentDefinition,
    ContractContentDefinitionPublisherReference,
    ContractFriendly,
    ContractFriendlyContentReferenceReference,
    ContractLegal,
    ContractLegalContentReferenceReference,
    ContractLegallyBindingReferenceReference,
    ContractReference,
    ContractRule,
    ContractSigner,
    ContractSignerPartyReference,
    ContractSubjectReference,
    ContractTerm,
    ContractTermAction,
    ContractTermActionPerformerReference,
    ContractTermActionReasonCodeableReference,
    ContractTermActionReasonReference,
    ContractTermActionRequesterReference,
    ContractTermActionSubject,
    ContractTermActionSubjectReferenceReference,
    ContractTermAsset,
    ContractTermAssetContext,
    ContractTermAssetValuedItem,
    ContractTermAssetValuedItemRecipientReference,
    ContractTermAssetValuedItemResponsibleReference,
    ContractTermOffer,
    ContractTermOfferAnswer,
    ContractTermOfferParty,
    ContractTermOfferPartyReferenceReference,
    ContractTermSecurityLabel,
)


class ContractSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Subject Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractSubjectReference
        exclude = ["created_at", "updated_at"]

    # def to_internal_value(self, data):
    #     """To internal value."""
    #     breakpoint()
    #     x = super().to_internal_value(data)
    #     return x


class ContractReferenceSerializer(BaseReferenceModelSerializer):
    """Contract reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractReference
        exclude = ["created_at", "updated_at"]


class ContractRuleSerializer(WritableNestedModelSerializer):
    """Contract Rule serializer."""

    content_attachment = AttachmentSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractRule
        exclude = ["created_at", "updated_at"]


class ContractLegalContentReferenceReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Legal Content Reference Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractLegalContentReferenceReference
        exclude = ["created_at", "updated_at"]


class ContractLegalSerializer(WritableNestedModelSerializer):
    """Contract Legal serializer."""

    content_attachment = AttachmentSerializer(many=False, required=False)
    content_reference = ContractLegalContentReferenceReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ContractLegal
        exclude = ["created_at", "updated_at"]


class ContractSignerPartyReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Signer Party Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractSignerPartyReference
        exclude = ["created_at", "updated_at"]


class ContractSignerSerializer(WritableNestedModelSerializer):
    """Contract Signer serializer."""

    type = CodingSerializer(many=False, required=False)
    party = ContractSignerPartyReferenceSerializer(many=False, required=False)
    signature = SignatureSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ContractSigner
        exclude = ["created_at", "updated_at"]


class ContractFriendlyContentReferenceReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Friendly Content Reference Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractFriendlyContentReferenceReference
        exclude = ["created_at", "updated_at"]


class ContractFriendlySerializer(WritableNestedModelSerializer):
    """Contract Friendly serializer."""

    content_attachment = AttachmentSerializer(many=False, required=False)
    content_reference = ContractFriendlyContentReferenceReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ContractFriendly
        exclude = ["created_at", "updated_at"]


class ContractTermOfferPartyReferenceReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Term Offer Party Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermOfferPartyReferenceReference
        exclude = ["created_at", "updated_at"]


class ContractTermOfferPartySerializer(WritableNestedModelSerializer):
    """Contract Term Offer Party serializer."""

    reference = ContractTermOfferPartyReferenceReferenceSerializer(
        many=True, required=False
    )
    role = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermOfferParty
        exclude = ["created_at", "updated_at"]


class ContractTermOfferAnswerSerializer(WritableNestedModelSerializer):
    """Contract Term Offer Answer serializer."""

    value_attachment = AttachmentSerializer(many=False, required=False)
    value_coding = CodingSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_reference = ReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermOfferAnswer
        exclude = ["created_at", "updated_at"]


class ContractTermOfferSerializer(WritableNestedModelSerializer):
    """Contract Term Offer serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    party = ContractTermOfferPartySerializer(many=True, required=False)
    topic = ReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    decision = CodeableConceptSerializer(many=False, required=False)
    decision_mode = CodeableConceptSerializer(many=True, required=False)
    answer = ContractTermOfferAnswerSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermOffer
        exclude = ["created_at", "updated_at"]


class ContractTermAssetContextSerializer(WritableNestedModelSerializer):
    """Contract Term Asset Context serializer."""

    reference = ReferenceSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermAssetContext
        exclude = ["created_at", "updated_at"]


class ContractTermAssetValuedItemResponsibleReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Contract Term Asset Valued Item Responsible Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermAssetValuedItemResponsibleReference
        exclude = ["created_at", "updated_at"]


class ContractTermAssetValuedItemRecipientReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Contract Term Asset Valued Item Recipient Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermAssetValuedItemRecipientReference
        exclude = ["created_at", "updated_at"]


class ContractTermAssetValuedItemSerializer(WritableNestedModelSerializer):
    """Contract Term Asset Valued Item serializer."""

    entity_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    entity_reference = ReferenceSerializer(many=False, required=False)
    identifier = IdentifierSerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    responsible = ContractTermAssetValuedItemResponsibleReferenceSerializer(
        many=False, required=False
    )
    recipient = ContractTermAssetValuedItemRecipientReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ContractTermAssetValuedItem
        exclude = ["created_at", "updated_at"]


class ContractTermAssetSerializer(WritableNestedModelSerializer):
    """Contract Term Asset serializer."""

    scope = CodeableConceptSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    type_reference = ReferenceSerializer(many=True, required=False)
    subtype = CodeableConceptSerializer(many=True, required=False)
    relationship = CodingSerializer(many=False, required=False)
    context = ContractTermAssetContextSerializer(many=True, required=False)
    period_type = CodeableConceptSerializer(many=True, required=False)
    period = PeriodSerializer(many=True, required=False)
    use_period = PeriodSerializer(many=True, required=False)
    answer = ContractTermOfferAnswerSerializer(many=True, required=False)
    valued_item = ContractTermAssetValuedItemSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermAsset
        exclude = ["created_at", "updated_at"]


class ContractTermActionSubjectReferenceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Contract Term Action Subject Reference Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermActionSubjectReferenceReference
        exclude = ["created_at", "updated_at"]


class ContractTermActionSubjectSerializer(WritableNestedModelSerializer):
    """Contract Term Action Subject serializer."""

    reference = ContractTermActionSubjectReferenceReferenceSerializer(
        many=True, required=False
    )
    role = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermActionSubject
        exclude = ["created_at", "updated_at"]


class ContractTermActionRequesterReferenceSerializer(WritableNestedModelSerializer):
    """Contract Term Action Requester Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermActionRequesterReference
        exclude = ["created_at", "updated_at"]


class ContractTermActionPerformerReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Term Action Performer Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermActionPerformerReference
        exclude = ["created_at", "updated_at"]


class ContractTermActionReasonReferenceSerializer(WritableNestedModelSerializer):
    """Contract Term Action Reason Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermActionReasonReference
        exclude = ["created_at", "updated_at"]


class ContractTermActionReasonCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Contract Term Action Reason Codeable Reference serializer."""

    reference = ContractTermActionReasonReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermActionReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class ContractTermActionSerializer(WritableNestedModelSerializer):
    """Contract Term Action serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    subject = ContractTermActionSubjectSerializer(many=True, required=False)
    intent = CodeableConceptSerializer(many=False, required=False)
    status = CodeableConceptSerializer(many=False, required=False)
    context = EncounterEpisodeOfCareReferenceSerializer(many=False, required=False)
    occurrence_period = PeriodSerializer(many=False, required=False)
    occurrence_timing = TimingSerializer(many=False, required=False)
    requester = ContractTermActionRequesterReferenceSerializer(
        many=False, required=False
    )
    performer_type = CodeableConceptSerializer(many=True, required=False)
    performer_role = CodeableConceptSerializer(many=False, required=False)
    performer = ContractTermActionPerformerReferenceSerializer(
        many=False, required=False
    )
    reason = ContractTermActionReasonCodeableReferenceSerializer(
        many=True, required=False
    )
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermAction
        exclude = ["created_at", "updated_at"]


class ContractAuthorReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Author Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractAuthorReference
        exclude = ["created_at", "updated_at"]


class ContractContentDefinitionPublisherReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Contract Content Definition Publisher Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractContentDefinitionPublisherReference
        exclude = ["created_at", "updated_at"]


class ContractContentDefinitionSerializer(WritableNestedModelSerializer):
    """Contract Content Definition serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    sub_type = CodeableConceptSerializer(many=False, required=False)
    publisher = ContractContentDefinitionPublisherReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ContractContentDefinition
        exclude = ["created_at", "updated_at"]


class ContractTermSecurityLabelSerializer(WritableNestedModelSerializer):
    """Contract Term Security Label serializer."""

    classification = CodingSerializer(many=False, required=False)
    category = CodingSerializer(many=True, required=False)
    control = CodingSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ContractTermSecurityLabel
        exclude = ["created_at", "updated_at"]


class ContractLegallyBindingReferenceReferenceSerializer(BaseReferenceModelSerializer):
    """Contract Legally Binding Reference Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ContractLegallyBindingReferenceReference
        exclude = ["created_at", "updated_at"]


class ContractTermSerializer(WritableNestedModelSerializer):
    """Contract Term serializer."""

    identifier = IdentifierSerializer(many=False, required=False)
    applies = PeriodSerializer(many=False, required=False)
    topic_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    topic_reference = ReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    sub_type = CodeableConceptSerializer(many=False, required=False)
    security_label = ContractTermSecurityLabelSerializer(many=False, required=False)
    offer = ContractTermOfferSerializer(many=False, required=False)
    asset = ContractTermAssetSerializer(many=True, required=False)
    action = ContractTermActionSerializer(many=True, required=False)
    group = RecursiveField(
        "ContractTermSerializer", many=True, required=False
    )  # TODO need to fix this as it currently throws an error.

    class Meta:
        """Meta class."""

        model = ContractTerm
        exclude = ["created_at", "updated_at"]


class ContractSerializer(BaseWritableNestedModelSerializer):
    """Contract serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    legal_state = CodeableConceptSerializer(many=False, required=False)
    instantiates_canonical = ContractReferenceSerializer(many=False, required=False)
    content_derivative = CodeableConceptSerializer(many=False, required=False)
    applies = PeriodSerializer(many=False, required=False)
    expiration_type = CodeableConceptSerializer(many=False, required=False)
    subject = ContractSubjectReferenceSerializer(many=True, required=False)
    authority = OrganizationReferenceSerializer(many=True, required=False)
    domain = LocationReferenceSerializer(many=True, required=False)
    author = ContractAuthorReferenceSerializer(many=False, required=False)
    scope = CodeableConceptSerializer(many=False, required=False)
    topic_codeable_concept = CodeableConceptSerializer(many=True, required=False)
    topic_reference = ReferenceSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    sub_type = CodeableConceptSerializer(many=True, required=False)
    content_definition = ContractContentDefinitionSerializer(many=False, required=False)
    term = ContractTermSerializer(many=True, required=False)
    supporting_info = ReferenceSerializer(many=True, required=False)
    relevant_history = ProvenanceReferenceSerializer(many=True, required=False)
    signer = ContractSignerSerializer(many=True, required=False)
    friendly = ContractFriendlySerializer(many=True, required=False)
    legal = ContractLegalSerializer(many=True, required=False)
    rule = ContractRuleSerializer(many=True, required=False)
    legally_binding_attachment = AttachmentSerializer(many=False, required=False)
    legally_binding_reference = ContractLegallyBindingReferenceReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = Contract
        exclude = ["created_at", "updated_at"]



"""Device choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DeviceMetricOperationalStatus(models.TextChoices):
    """Device metric operational status choices."""

    ON = "on", _("on")
    OFF = "off", _("off")
    STANDBY = "standby", _("standby")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class DeviceMetricCalibrationState(models.TextChoices):
    """Device metric calibration state choices."""

    NOT_CALIBRATED = "not-calibrated", _("Not Calibrated")
    CALIBRATION_REQUIRED = "calibration-required", _("Calibration Required")
    CALIBRATED = "calibrated", _("Calibrated")
    UNSPECIFIED = "unspecified", _("Unspecified")



"""Invoice models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    MonetaryComponent,
    Money,
    OrganizationReference,
    Period,
    TimeStampedModel,
)

from . import choices


class InvoiceRecipientReference(BaseReference):
    """Invoice recipient reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="invoice_recipient_reference_identifier",
        null=True,
    )
    organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="invoice_recipient_reference_organization",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="invoice_recipient_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="invoice_recipient_reference_related_person",
        null=True,
    )


class InvoiceParticipantActorReference(BaseReference):
    """Invoice participant actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_practitioner",
        null=True,
    )
    organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_organization",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_patient",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_practitioner_role",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_device",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor_reference_related_person",
        null=True,
    )


class InvoiceParticipant(TimeStampedModel):
    """Invoice participant model."""

    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="invoice_participant_role",
        null=True,
    )
    actor = models.ForeignKey(
        InvoiceParticipantActorReference,
        on_delete=models.CASCADE,
        related_name="invoice_participant_actor",
        null=True,
    )


class InvoiceLineItem(TimeStampedModel):
    """Invoice line item model."""

    sequence = models.PositiveIntegerField(null=True)
    service_date = models.DateField(null=True)
    service_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="invoice_line_item_service_period",
        null=True,
    )
    charge_item_reference = models.ForeignKey(
        "chargeitems.ChargeItemReference",
        on_delete=models.CASCADE,
        related_name="invoice_line_item_charge_item_reference",
        null=True,
    )
    charge_item_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="invoice_line_item_charge_item_codeable_concept",
        null=True,
    )
    price_component = models.ManyToManyField(
        MonetaryComponent,
        related_name="invoice_line_item_price_component",
        blank=True,
    )


class Invoice(TimeStampedModel):
    """Invoice model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="invoice_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255,
        choices=choices.InvoiceStatus.choices,
        default=choices.InvoiceStatus.DRAFT,
    )
    cancelled_reason = models.TextField(null=True)
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="invoice_type",
        null=True,
    )
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.CASCADE,
        related_name="invoice_subject",
        null=True,
    )
    recipient = models.ForeignKey(
        InvoiceRecipientReference,
        on_delete=models.CASCADE,
        related_name="invoice_recipient",
        null=True,
    )
    date = models.DateTimeField(null=True)
    creation = models.DateTimeField(null=True)
    period_date = models.DateField(null=True)
    period_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="invoice_period_period",
        null=True,
    )
    participant = models.ManyToManyField(
        InvoiceParticipant,
        related_name="invoice_participant",
        blank=True,
    )
    issuer = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="invoice_issuer",
        null=True,
    )
    account = models.ForeignKey(
        "accounts.AccountReference",
        on_delete=models.CASCADE,
        related_name="invoice_account",
        null=True,
    )
    line_item = models.ManyToManyField(
        InvoiceLineItem,
        related_name="invoice_line_item",
        blank=True,
    )
    total_price_component = models.ManyToManyField(
        MonetaryComponent,
        related_name="invoice_total_price_component",
        blank=True,
    )
    total_net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="invoice_total_net",
        null=True,
    )
    total_gross = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="invoice_total_gross",
        null=True,
    )
    payment_terms = models.TextField(null=True)
    note = models.ManyToManyField(
        Annotation,
        related_name="invoice_note",
        blank=True,
    )


class InvoiceReference(BaseReference):
    """Invoice reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="invoice_reference_identifier",
        null=True,
    )
    invoice = models.ForeignKey(
        Invoice,
        on_delete=models.CASCADE,
        related_name="invoice_reference_invoice",
        null=True,
    )



"""Deviceassociations models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    TimeStampedModel,
)


class DeviceAssociationOperationOperatorReference(BaseReference):
    """DeviceAssociationOperationOperatorReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="device_association_operation_operator_reference_identifier",
    )

    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.SET_NULL,
        null=True,
        related_name="device_association_operation_operator_reference_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.SET_NULL,
        null=True,
        related_name="device_association_operation_operator_reference_practitioner",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.SET_NULL,
        null=True,
        related_name="device_association_operation_operator_reference_related_person",
    )


class DeviceAssociationOperation(TimeStampedModel):
    """DeviceAssociationOperation model."""

    status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="device_association_operation_status",
    )
    operator = models.ManyToManyField(
        DeviceAssociationOperationOperatorReference,
        related_name="device_association_operation_operator",
        blank=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        null=True,
        related_name="device_association_operation_period",
    )


class DeviceAssociationSubjectReference(BaseReference):
    """DeviceAssociationSubjectReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        null=True,
        related_name="device_association_subject_reference_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_association_subject_reference_patient",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_association_subject_reference_group",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_association_subject_reference_practitioner",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_association_subject_reference_related_person",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_association_subject_reference_device",
    )


class DeviceAssociation(TimeStampedModel):
    """DeviceAssociation model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="device_association_identifier",
        blank=True,
    )
    device = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.CASCADE,
        related_name="device_association_device",
    )
    relationship = models.ManyToManyField(
        CodeableConcept,
        related_name="device_association_relationship",
        blank=True,
    )
    status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="device_association_status",
        null=True,
    )
    status_reason = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="device_association_status_reason",
    )
    subject = models.ForeignKey(
        DeviceAssociationSubjectReference,
        on_delete=models.CASCADE,
        related_name="device_association_subject",
    )
    body_structure = models.ForeignKey(
        "bodystructures.BodyStructureReference",
        on_delete=models.CASCADE,
        null=True,
        related_name="device_association_body_structure",
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        null=True,
        related_name="device_association_period",
    )
    operation = models.ManyToManyField(
        DeviceAssociationOperation,
        related_name="device_association_operation",
        blank=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('careteams', '0004_initial'),
        ('conditions', '0004_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0003_initial'),
        ('groups', '0003_initial'),
        ('nutritionintakes', '0002_initial'),
        ('nutritionproducts', '0001_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='nutritionintakebasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_based_on_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='nutritionintake',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_based_on', to='nutritionintakes.nutritionintakebasedonreference'),
        ),
        migrations.AddField(
            model_name='nutritionintakecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitem',
            name='not_consumed_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_not_consumed_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitem',
            name='nutrition_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_nutrition_product', to='nutritionproducts.nutritionproductcodeablereference'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitem',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='nutritionintake',
            name='nutrition_item',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_nutrition_item', to='nutritionintakes.nutritionintakenutritionitem'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemconsumeditem',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_consumed_item_amount', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemconsumeditem',
            name='rate_quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_consumed_item_rate_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemconsumeditem',
            name='rate_ratio',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_consumed_item_rate_ratio', to='base.ratio'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemconsumeditem',
            name='schedule',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_consumed_item_schedule', to='base.timing'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitem',
            name='consumed_item',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_nutrition_item_consumed_item', to='nutritionintakes.nutritionintakenutritionitemconsumeditem'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemtotalintake',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_total_intake_amount', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemtotalintake',
            name='energy',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_total_intake_energy', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemtotalintake',
            name='nutrient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_nutrition_item_total_intake_nutrient', to='substances.substancecodeablereference'),
        ),
        migrations.AddField(
            model_name='nutritionintakenutritionitemconsumeditem',
            name='total_intake',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_nutrition_item_consumed_item_total_intake', to='nutritionintakes.nutritionintakenutritionitemtotalintake'),
        ),
        migrations.AddField(
            model_name='nutritionintakepartofreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_part_of_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='nutritionintakepartofreference',
            name='nutrition_intake',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_part_of_reference_nutrition_intake', to='nutritionintakes.nutritionintake'),
        ),
        migrations.AddField(
            model_name='nutritionintakepartofreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_part_of_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='nutritionintakepartofreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_part_of_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='nutritionintake',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_part_of', to='nutritionintakes.nutritionintakepartofreference'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformer',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_reference_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='nutritionintake',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_performer', to='nutritionintakes.nutritionintakeperformer'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformeractorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_actor_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='nutritionintakeperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_performer_reference_actor', to='nutritionintakes.nutritionintakeperformeractorreference'),
        ),
        migrations.AddField(
            model_name='nutritionintakereasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reason_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='nutritionintake',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='nutrition_intake_reason_code', to='nutritionintakes.nutritionintakereasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='nutritionintakereasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='nutritionintakereasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='nutritionintakereasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='nutritionintakereasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='nutritionintakereasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_codeable_reference_reference', to='nutritionintakes.nutritionintakereasonreference'),
        ),
        migrations.AddField(
            model_name='nutritionintakereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='nutritionintakereference',
            name='nutrition_intake',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reference_nutrition_intake', to='nutritionintakes.nutritionintake'),
        ),
        migrations.AddField(
            model_name='nutritionintakecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_codeable_reference_reference', to='nutritionintakes.nutritionintakereference'),
        ),
        migrations.AddField(
            model_name='nutritionintakereportedreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='nutritionintakereportedreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='nutritionintakereportedreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='nutritionintakereportedreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='nutritionintakereportedreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='nutritionintakereportedreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='nutritionintake',
            name='reported_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_intake_reported_reference', to='nutritionintakes.nutritionintakereportedreference'),
        ),
    ]



"""Vision prescription choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class VisionPrescriptionStatusChoices(models.TextChoices):
    """Vision prescription status choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class VisionPrescriptionLensSpecificationEyeChoices(models.TextChoices):
    """Vision prescription lens specification eye choices."""

    RIGHT = "right", _("Right")
    LEFT = "left", _("Left")


class VisionPrescriptionLensSpecificationPrismBaseChoices(models.TextChoices):
    """Vision prescription lens specification prism base choices."""

    UP = "up", _("Up")
    DOWN = "down", _("Down")
    IN = "in", _("In")
    OUT = "out", _("Out")



"""Medication Knowledge models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    TimeStampedModel,
)
from dfhir.medicationknowledges.choices import MedicationKnowledgeStatusChoices


class MedicationKnowledgeReference(BaseReference):
    """Medication Knowledge Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_reference_identifier",
    )
    medication_knowledge = models.ForeignKey(
        "MedicationKnowledge",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_reference_medication_knowledge",
    )


class MedicationKnowledgeRelatedMedicationKnowledge(TimeStampedModel):
    """Medication Knowledge Related Medication Knowledge model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_related_medication_knowledge_type",
    )
    reference = models.ForeignKey(
        MedicationKnowledgeReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_related_medication_knowledge_reference",
    )


class MedicationKnowledgeMonograph(TimeStampedModel):
    """Medication Knowledge Monograph model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_monograph_type",
    )
    source = models.ForeignKey(
        "documentreferences.DocumentReferenceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_monograph_source",
    )


class MedicationKnowledgeCost(TimeStampedModel):
    """medication knowledge cost model."""

    effective_date = models.ManyToManyField(
        Period, blank=True, related_name="medication_knowledge_cost_effective_date"
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_cost_type",
    )
    source = models.CharField(max_length=255, null=True)
    cost_money = models.ForeignKey(
        "base.Money",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_cost_cost_money",
    )
    cost_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_cost_cost_codeable_concept",
    )


class MedicationKnowledgeMonitoringProgram(TimeStampedModel):
    """Medication Knowledge Monitoring Program model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_monitoring_program_type",
    )
    name = models.CharField(max_length=255, null=True)


class MedicationKnowledgePatientCharacteristics(TimeStampedModel):
    """medication knowledge dosing guideline treatment intent dosage patient characteristics model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_type",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_value_codeable_concept",
    )
    value_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_value_quantity",
    )
    value_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="med_knowledge_dosing_guideline_treatment_intent_dosage_patient_characteristics_value_range",
    )


class MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage(TimeStampedModel):
    """medication knowledge dosing guideline treatment intent dosage model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_dosing_guideline_dosage_type",
    )
    # TODO: dosage = models.ForeignKey(
    #     "Dosage",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="medication_knowledge_dosing_guideline_dosage_dose_quantity",
    # )


class MedicationKnowledgeIndicationGuidelineDosingGuideline(TimeStampedModel):
    """medication knowledge dosing guideline model."""

    treatment_intent = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_dosing_guideline_treatment_intent",
    )
    dosage = models.ManyToManyField(
        MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage,
        blank=True,
        related_name="medication_knowledge_dosing_guideline_dose",
    )
    administration_treatment = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_dosing_guideline_administration_treatment",
    )
    patient_characteristics = models.ManyToManyField(
        MedicationKnowledgePatientCharacteristics,
        blank=True,
        related_name="medication_knowledge_dosing_guideline_patient_characteristics",
    )


class MedicationKnowledgeIndicationGuideline(TimeStampedModel):
    """medication knowledge indication guideline model."""

    # TODO: indication = models.ManyToManyField(
    #     "ClinicalUseDefinition",
    #     blank=True,
    #     related_name="medication_knowledge_indication_guidline_indication",
    # )
    dosing_guideline = models.ManyToManyField(
        MedicationKnowledgeIndicationGuidelineDosingGuideline,
        blank=True,
        related_name="medication_knowledge_indication_guidline_dosing_guideline",
    )


class MedicationKnowledgeMedicineClassification(TimeStampedModel):
    """medication knowledge medicine classification model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_medicine_classification_type",
    )
    source_string = models.CharField(max_length=255, null=True)
    source_uri = models.URLField(null=True)
    classification = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="medication_knowledge_medicine_classification_classification",
    )


class MedicationKnowledgePackaging(TimeStampedModel):
    """medication knowledge packaging model."""

    cost = models.ForeignKey(
        MedicationKnowledgeCost,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_packaging_cost",
    )
    # TODO: pockage_product = models.ForeignKey(
    #     "PackageProductDefinitionReference", on_delete=models.DO_NOTHING, null=True
    # )


class MedicationKnowledgeStorageGuidelineEnvironmentalSetting(TimeStampedModel):
    """medication knowledge storage guideline environmental setting model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
    )
    value_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="storage_guideline_environmental_setting_value_quantity",
    )
    value_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="storage_guideline_environmental_setting_value_range",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="storage_guideline_environmental_setting_value_codeable_concept",
    )


class MedicationKnowledgeStorageGuideline(TimeStampedModel):
    """medication knowledge storage guideline model."""

    reference = models.URLField(null=True)
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="storage_guideline_note"
    )
    stability_duration = models.ForeignKey(
        "base.Duration",
        on_delete=models.DO_NOTHING,
        related_name="storage_guideline_stability_duration",
        null=True,
    )
    environmental_setting = models.ManyToManyField(
        MedicationKnowledgeStorageGuidelineEnvironmentalSetting,
        blank=True,
        related_name="storage_guideline_environmental_setting",
    )


class MedicationKnowledgeRegulatorySubstitution(TimeStampedModel):
    """Medication Knowledge Regulatory Substitution model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="regulatory_substitution_type",
    )
    allowed = models.BooleanField(null=True)


class MedicationKnowledgeRegulatoryMaxDispense(TimeStampedModel):
    """Medication Knowledge Regulatory Max Dispense model."""

    quantity = models.ForeignKey(
        "base.SimpleQuantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="regulatory_max_dispense_quantity",
    )
    period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="regulatory_max_dispense_period",
    )


class MedicationKnowledgeRegulatory(TimeStampedModel):
    """Medication Knowledge Regulatory model."""

    regulatory_authority = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="regulatory_authority",
    )
    substitution = models.ManyToManyField(
        MedicationKnowledgeRegulatorySubstitution,
        blank=True,
        related_name="regulatory_substitution",
    )
    schedule = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="medication_knowledge_schedule"
    )
    max_dispense = models.ManyToManyField(
        MedicationKnowledgeRegulatoryMaxDispense,
        blank=True,
        related_name="regulatory_max_dispense",
    )


class MedicationKnowledgeDefinitionalIngredient(TimeStampedModel):
    """Medication Knowledge Definitional Ingredient model."""

    item = models.ForeignKey(
        "substances.SubstanceCodeableReference",
        on_delete=models.DO_NOTHING,
        related_name="medication_knowledge_definitional_ingredient_item",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        related_name="medication_knowledge_definitional_ingredient_type",
        null=True,
        on_delete=models.DO_NOTHING,
    )
    strength_ratio = models.ForeignKey(
        "base.Ratio",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_ingredient_strength_ratio",
    )
    strength_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_ingredient_strength_quantity",
    )
    strength_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_ingredient_strength_codeable_concept",
    )


class MedicationKnowledgeDefinitionalDrugCharacteristic(TimeStampedModel):
    """medication knowledge definitional drug characteristic model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_drug_characteristic_type",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_drug_characteristic_value_codeable_concept",
    )
    value_string = models.CharField(max_length=255, null=True)
    value_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_drug_characteristic_value_quantity",
    )
    value_base_64_binary = models.TextField(null=True)
    value_attachment = models.ForeignKey(
        "base.Attachment",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_drug_characteristic_value_attachment",
    )


class MedicationKnowledgeDefinitional(TimeStampedModel):
    """medication knowledge definitional model."""

    # TODO: definition = models.ManyToManyField(
    #     "MedicationalProductDefinitionReference",
    #     blank=True,
    #     related_name="medication_knowledge_definitional_definition",
    # )
    dose_form = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_knowledge_definitional_dose_form",
    )
    intended_route = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="medication_knowledge_definitional_intended_route",
    )
    ingredient = models.ManyToManyField(
        MedicationKnowledgeDefinitionalIngredient,
        blank=True,
        related_name="medication_knowledge_definitional_ingredient",
    )
    drug_characteristic = models.ManyToManyField(
        MedicationKnowledgeDefinitionalDrugCharacteristic,
        blank=True,
        related_name="medication_knowledge_definitional_drug_chracteristic",
    )


class MedicationKnowledge(TimeStampedModel):
    """medication knowledge model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="medication_knowledge_identifier"
    )
    code = models.ForeignKey(
        CodeableConcept,
        null=True,
        related_name="medication_knowledge_code",
        on_delete=models.DO_NOTHING,
    )
    status = models.CharField(
        max_length=255, null=True, choices=MedicationKnowledgeStatusChoices.choices
    )
    author = models.ForeignKey(
        "base.ContactDetail",
        null=True,
        related_name="medication_knowledge_author",
        on_delete=models.DO_NOTHING,
    )
    jurisdiction = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="medication_knowledge_jurisdiction"
    )
    name = models.CharField(max_length=255, null=True)
    related_medication_knowledge = models.ManyToManyField(
        MedicationKnowledgeRelatedMedicationKnowledge,
        blank=True,
        related_name="medication_knowledge_related_medication_knowledge",
    )
    associate_medication = models.ManyToManyField(
        "medications.MedicationReference",
        blank=True,
        related_name="medication_knowledge_associate_medication",
    )
    product_type = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="medication_knowledge_product_type",
    )
    monograph = models.ManyToManyField(
        MedicationKnowledgeMonograph,
        blank=True,
        related_name="medication_knowledge_monograph",
    )
    preparation_instruction = models.TextField(null=True)
    cost = models.ManyToManyField(
        MedicationKnowledgeCost, blank=True, related_name="medication_knowledge_cost"
    )
    monitoring_program = models.ManyToManyField(
        MedicationKnowledgeMonitoringProgram,
        blank=True,
        related_name="medication_knowledge_monitoring_program",
    )
    indication_guideline = models.ManyToManyField(
        MedicationKnowledgeIndicationGuideline,
        blank=True,
        related_name="medication_knowledge_indication_guideline",
    )
    medicine_classification = models.ManyToManyField(
        MedicationKnowledgeMedicineClassification,
        blank=True,
        related_name="medication_knowledge_medicine_classification",
    )
    packaging = models.ManyToManyField(
        MedicationKnowledgePackaging,
        blank=True,
        related_name="medication_knowledge_packaging",
    )
    # TODO: clinical_use_issue = models.ManyToManyField("ClinicalUseIssueReference")
    storage_guideline = models.ManyToManyField(
        MedicationKnowledgeStorageGuideline,
        blank=True,
        related_name="medication_knowledge_storage_guideline",
    )
    regulatory = models.ManyToManyField(
        MedicationKnowledgeRegulatory,
        blank=True,
        related_name="medication_knowledge_regulatory",
    )
    definitional = models.ForeignKey(
        MedicationKnowledgeDefinitional,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_knowledge_definitional",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('activitydefinitions', '0004_initial'),
        ('devicedefinitions', '0001_initial'),
        ('devices', '0001_initial'),
        ('endpoints', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='activitydefinitionparticipanttypereference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_device_reference_idevice', to='devices.device'),
        ),
        migrations.AddField(
            model_name='activitydefinitionparticipanttypereference',
            name='device_definition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_device_definition_reference_device_definition', to='devicedefinitions.devicedefinition'),
        ),
        migrations.AddField(
            model_name='activitydefinitionparticipanttypereference',
            name='endpoint',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_participant_type_reference_endpoint', to='endpoints.endpoint'),
        ),
    ]



"""Invoice urls."""

from django.urls import path

from . import views

app_name = "invoices"


urlpatterns = [
    path("invoices/", views.InvoiceListView.as_view()),
    path("invoices/<int:pk>/", views.InvoiceDetailView.as_view()),
]



"""Encounter filter module."""

from django_filters import DateTimeFromToRangeFilter
from django_filters import rest_framework as filters

from .models import Encounter


class EncounterFilter(filters.FilterSet):
    """Encounter filter."""

    id = filters.CharFilter(field_name="id")
    start_date_time = DateTimeFromToRangeFilter(field_name="start_date_time")
    end_date_time = DateTimeFromToRangeFilter(field_name="end_date_time")
    patient = filters.CharFilter(field_name="patient__id")
    practitioner = filters.CharFilter(
        field_name="encounter_participant__practitioner__id"
    )

    class Meta:
        """Meta class."""

        model = Encounter
        fields = ["id", "start_date_time", "end_date_time", "patient", "practitioner"]



"""Schedule model."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    TimeStampedModel,
)
from dfhir.healthcareservices.models import HealthCareServiceCodeableReference
from dfhir.practitioners.models import Practitioner


class SchedulesActorReference(BaseReference):
    """Actor model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="schedules_actor_identifier", blank=True
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="schedules_actor_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.DO_NOTHING,
        related_name="schedules_actor_practitioner",
        null=True,
    )
    healthcareservice = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        related_name="schedules_actor_healthcareservice",
        null=True,
    )


class Schedule(TimeStampedModel):
    """Schedule model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="schedule_identifier", blank=True
    )
    active = models.BooleanField(default=True)
    name = models.CharField(max_length=255, null=True)
    service_category = models.ManyToManyField(
        CodeableConcept, related_name="schedule_category"
    )
    specialty = models.ManyToManyField(
        CodeableConcept, related_name="schedule_specialty"
    )
    service_type = models.ManyToManyField(
        HealthCareServiceCodeableReference, related_name="schedule_servicetype"
    )
    slots_duration = models.IntegerField(default=30)
    planning_horizon = models.ForeignKey(Period, on_delete=models.DO_NOTHING, null=True)
    comment = models.TextField(null=True)
    actor = models.ManyToManyField(
        SchedulesActorReference, related_name="schedule_actor"
    )


class ScheduleReference(BaseReference):
    """ScheduleReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="schedule_reference_identifier",
        null=True,
    )
    schedule = models.ForeignKey(
        Schedule,
        on_delete=models.DO_NOTHING,
        related_name="schedule_reference_schedule",
        null=True,
    )



"""immunization recommendation serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    ReferenceSerializer,
)
from dfhir.immunizationrecommendations.models import (
    ImmunizationRecommendation,
    ImmunizationRecommendationRecommendation,
    ImmunizationRecommendationRecommendationDateCriterion,
    ImmunizationRecommendationSupportingImmunizationReference,
)
from dfhir.patients.serializers import PatientReferenceSerializer


class ImmunizationRecommendationSupportingImmunizationReferenceSerializer(
    BaseReferenceModelSerializer
):
    """immunization recommendation  supporting immunization reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta class."""

        model = ImmunizationRecommendationSupportingImmunizationReference
        exclude = ["created_at", "updated_at"]


class ImmunizationRecommendationRecommendationDateCriterionSerializer(
    WritableNestedModelSerializer
):
    """immunization recommendation recommendation date criterion serializer."""

    code = CodeableConceptSerializer(required=False)

    class Meta:
        """meta class."""

        model = ImmunizationRecommendationRecommendationDateCriterion
        exclude = ["created_at", "updated_at"]


class ImmunizationRecommendationRecommendationSerializer(WritableNestedModelSerializer):
    """immunization recommendation recommendation serializer."""

    vaccine_code = CodeableConceptSerializer(many=True, required=False)
    target_disease = CodeableConceptSerializer(many=True, required=False)
    contraindicated_vaccine_code = CodeableConceptSerializer(many=True, required=False)
    forecast_status = CodeableConceptSerializer(required=False)
    forecast_reason = CodeableConceptSerializer(many=True, required=False)
    date_criterion = ImmunizationRecommendationRecommendationDateCriterionSerializer(
        required=False, many=True
    )
    supporting_information = (
        ImmunizationRecommendationSupportingImmunizationReferenceSerializer(
            required=False
        )
    )
    supporting_patient_information = ReferenceSerializer(many=True, required=False)

    class Meta:
        """meta class."""

        model = ImmunizationRecommendationRecommendation
        exclude = ["created_at", "updated_at"]


class ImmunizationRecommendationSerializer(BaseWritableNestedModelSerializer):
    """immunization recommendation serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(many=False, required=True)
    authority = OrganizationReferenceSerializer(required=False)
    recommendation = ImmunizationRecommendationRecommendationSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = ImmunizationRecommendation
        exclude = ["created_at", "updated_at"]



"""Claims views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Claim
from .serializers import ClaimSerializer


class ClaimListView(APIView):
    """Claim list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ClaimSerializer(many=True)})
    def get(self, request):
        """Get a list of claims."""
        claims = Claim.objects.all()
        serializer = ClaimSerializer(claims, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ClaimSerializer, responses={201: ClaimSerializer})
    def post(self, request):
        """Create a claim."""
        serializer = ClaimSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ClaimDetailView(APIView):
    """Claim detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a claim object."""
        try:
            return Claim.objects.get(pk=pk)
        except Claim.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ClaimSerializer})
    def get(self, request, pk=None):
        """Get a claim."""
        claim = self.get_object(pk)
        serializer = ClaimSerializer(claim)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ClaimSerializer, responses={200: ClaimSerializer})
    def patch(self, request, pk=None):
        """Update a claim."""
        claim = self.get_object(pk)
        serializer = ClaimSerializer(claim, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a claim."""
        claim = self.get_object(pk)
        claim.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""medication knowledge views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.medicationknowledges.models import MedicationKnowledge
from dfhir.medicationknowledges.serializers import MedicationKnowledgeSerializer


class MedicationKnowledgeListView(APIView):
    """MedicationKnowledge list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: MedicationKnowledgeSerializer(many=True)})
    def get(self, request):
        """Get a list of medication knowledge."""
        medication_knowledges = MedicationKnowledge.objects.all()
        serializer = MedicationKnowledgeSerializer(medication_knowledges, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=MedicationKnowledgeSerializer,
        responses={201: MedicationKnowledgeSerializer},
    )
    def post(self, request):
        """Create a medication knowledge."""
        serializer = MedicationKnowledgeSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MedicationKnowledgeDetailView(APIView):
    """medication knowledge detail view."""

    def get_object(self, pk):
        """Get medication knowledge object."""
        try:
            return MedicationKnowledge.objects.get(pk=pk)
        except MedicationKnowledge.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: MedicationKnowledgeSerializer})
    def get(self, request, pk=None):
        """Get a medication knowledge."""
        medication_knowledge = self.get_object(pk)
        serializer = MedicationKnowledgeSerializer(medication_knowledge)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MedicationKnowledgeSerializer,
        responses={200: MedicationKnowledgeSerializer},
    )
    def patch(self, request, pk=None):
        """Update a medication knowledge."""
        medication_knowledge = self.get_object(pk)
        serializer = MedicationKnowledgeSerializer(
            medication_knowledge, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a medication knowledge."""
        medication_knowledge = self.get_object(pk)
        medication_knowledge.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""observation views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import (
    Observation,
)
from .serializers import (
    ObservationSerializer,
)


class ObservationListView(APIView):
    """observation list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200, ObservationSerializer(many=True)})
    def get(self, request):
        """Get all observations."""
        observation = Observation.objects.all()
        serializer = ObservationSerializer(observation, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ObservationSerializer, responses={201, ObservationSerializer}
    )
    def post(self, request):
        """Create observation."""
        serializer = ObservationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ObservationDetailView(APIView):
    """observation detail view."""

    def get_object(self, pk):
        """Get an observation object by ID."""
        try:
            return Observation.objects.get(pk=pk)
        except Observation.DoesNotExist as error:
            raise Http404 from error

    @extend_schema(responses={200, ObservationSerializer})
    def get(self, request, pk=None):
        """Get an observation."""
        observation = self.get_object(pk)
        serializer = ObservationSerializer(observation)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200, ObservationSerializer})
    def patch(self, request, pk):
        """Patch an observation."""
        observation = self.get_object(pk)
        serializer = ObservationSerializer(observation, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk):
        """Delete an observation."""
        observation = self.get_object(pk)
        observation.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Organization model."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    CodeableConcept,
    ExtendedContactDetail,
    Identifier,
    OrganizationReference,
    Qualification,
    TimeStampedModel,
)
from dfhir.endpoints.models import EndpointReference

from .choices import OrganizationStatus


class Organization(TimeStampedModel):
    """Organization model."""

    name = models.CharField(max_length=255)
    identifier = models.ManyToManyField(
        Identifier, related_name="organization_identifier", blank=True
    )
    alias = ArrayField(models.CharField(max_length=255), null=True)
    website = models.URLField(null=True)
    email = models.EmailField(null=True)
    status = models.CharField(
        max_length=50,
        choices=OrganizationStatus.choices,
        default=OrganizationStatus.PENDING,
    )
    active = models.BooleanField(default=True)
    type = models.ManyToManyField(
        CodeableConcept, related_name="organization_type", blank=True
    )
    part_of = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        null=True,
        blank=True,
        related_name="organization_part_of",
    )
    description = models.TextField(null=True)
    qualification = models.ManyToManyField(
        Qualification, related_name="organization_qualification", blank=True
    )
    contact = models.ManyToManyField(
        ExtendedContactDetail, related_name="organization_contact", blank=True
    )
    endpoint = models.ManyToManyField(
        EndpointReference, related_name="organization_endpoint", blank=True
    )


class OrganizationCodeableReference(TimeStampedModel):
    """organization codeable concept model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="organization_codeable_concept_concept",
    )
    reference = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="organization_codeable_concept_reference",
    )



"""service requests urls."""

from django.urls import path

from . import views

app_name = "servicerequests"

urlpatterns = [
    path("servicerequests/", views.ServiceRequestListView.as_view(), name="list_view"),
    path(
        "servicerequests/<int:pk>/",
        views.ServiceRequestDetailView.as_view(),
        name="detail_view",
    ),
    path("servicerequests/reason/", views.ProcedureReasonListView.as_view()),
    path("servicerequests/bodysite/", views.BodySiteListView.as_view()),
    path("servicerequests/code/", views.ProcedureCodesListView.as_view()),
    path("servicerequests/asneeded/", views.AsNeededListView.as_view()),
    path("servicerequests/category/", views.ServiceRequestCategoryListView.as_view()),
    path("servicerequests/reference/", views.ReferenceListView.as_view()),
    path("servicerequests/parameter/", views.ParameterListView.as_view()),
]



"""Device usage models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    TimeStampedModel,
    Timing,
)

from . import choices


class DeviceUsageDerivedFromReference(BaseReference):
    """Device Usage Derived From Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_derived_from_reference_identifier",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_derived_from_reference_service_request",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_derived_from_reference_procedure",
        null=True,
    )
    claim = models.ForeignKey(
        "claims.Claim",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_derived_from_reference_claim",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_derived_from_reference_observation",
        null=True,
    )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.DO_NOTHING,
    #     related_name="device_usage_derived_from_reference_questionnaire_response",
    #     null=True,
    # )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_derived_from_reference_document_reference",
        null=True,
    )


class DeviceUsageAdherence(TimeStampedModel):
    """Device Usage Adherence model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_adherence_code",
        null=True,
    )
    reason = models.ManyToManyField(
        CodeableConcept, related_name="device_usage_adherence_reason", blank=True
    )


class DeviceUsageInformationSourceReference(BaseReference):
    """Device usage information source reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="device_usage_information_source_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_information_source_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_information_source_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_information_source_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_information_source_reference_related_person",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_information_source_reference_organization",
        null=True,
    )


class DeviceUsageReasonReference(BaseReference):
    """Device usage reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_reason_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_reason_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_reason_reference_observation",
        null=True,
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_reason_reference_diagnostic_report",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_reason_reference_document_reference",
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_reason_reference_procedure",
        null=True,
    )


class DeviceUsageReasonCodeableReference(TimeStampedModel):
    """Device usage reason codeable reference model."""

    reference = models.ForeignKey(
        DeviceUsageReasonReference,
        on_delete=models.CASCADE,
        null=True,
        related_name="device_usage_reason_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="device_usage_reason_codeable_reference_concept",
    )


class DeviceUsage(TimeStampedModel):
    """Device usage model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="device_usages_identifier", blank=True
    )
    based_on = models.ManyToManyField(
        "servicerequests.ServiceRequestReference",
        related_name="device_usage_based_on",
        blank=True,
    )
    status = models.CharField(
        max_length=255,
        choices=choices.DeviceUsageStatus.choices,
        default=choices.DeviceUsageStatus.ACTIVE,
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="device_usage_category", blank=True
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_patient",
        null=True,
    )
    derived_from = models.ManyToManyField(
        DeviceUsageDerivedFromReference,
        related_name="device_usage_derived_from",
        blank=True,
    )
    context = models.ForeignKey(
        "encounters.EncounterEpisodeOfCareReference",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_context",
        null=True,
    )
    timing_timing = models.ForeignKey(
        Timing,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_timing_timing",
        null=True,
    )
    timing_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_timing_period",
        null=True,
    )
    timing_date_time = models.DateTimeField(null=True)
    date_asserted = models.DateTimeField(null=True)
    usage_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_usage_status",
        null=True,
    )
    usage_reason = models.ManyToManyField(
        CodeableConcept, related_name="device_usage_usage_reason", blank=True
    )
    adherence = models.ForeignKey(
        DeviceUsageAdherence,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_adherence",
        null=True,
    )
    information_source = models.ForeignKey(
        DeviceUsageInformationSourceReference,
        on_delete=models.DO_NOTHING,
        related_name="device_usage_information_source",
        null=True,
    )
    device = models.ForeignKey(
        "devices.DeviceDeviceDefinitionCodeableReference",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_device",
        null=True,
    )
    reason = models.ManyToManyField(
        DeviceUsageReasonCodeableReference,
        related_name="device_usage_reason",
        blank=True,
    )
    body_site = models.ForeignKey(
        "bodystructures.BodyStructureCodeableReference",
        on_delete=models.DO_NOTHING,
        related_name="device_usage_body_site",
        null=True,
    )
    note = models.ManyToManyField(
        Annotation, related_name="device_usage_note", blank=True
    )



"""Claims serializers."""

from drf_writable_nested import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AddressSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    ReferenceSerializer,
    SimpleQuantitySerializer,
)
from dfhir.bodystructures.serializers import BodyStructureCodeableReferenceSerializer
from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.devices.serializers import DeviceReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import (
    LocationOrganizationReferenceSerializer,
    LocationReferenceSerializer,
)

# from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.procedures.serializers import ProcedureReferenceSerializer
from dfhir.servicerequests.serializers import (
    ServiceRequestReferenceSerializer,
)

from .models import (
    Claim,
    ClaimAccident,
    ClaimCareTeam,
    ClaimCareTeamProviderReference,
    ClaimDiagnosis,
    ClaimEntererReference,
    ClaimEvent,
    ClaimInsurance,
    ClaimItem,
    ClaimItemBodySIte,
    ClaimItemDetail,
    ClaimItemDetailSubDetail,
    ClaimItemRequestReference,
    ClaimOriginalPrescriptionReference,
    ClaimPayee,
    ClaimPayeePartyReference,
    ClaimPrescriptionReference,
    ClaimProcedure,
    ClaimProviderReference,
    ClaimReference,
    ClaimRelated,
    ClaimSupportingInfo,
)


class ClaimReferenceSerializer(BaseReferenceModelSerializer):
    """Claim reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimReference
        exclude = ["created_at", "updated_at"]


class ClaimPayeePartyReferenceSerializer(BaseReferenceModelSerializer):
    """Claim Payee Party Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimPayeePartyReference
        exclude = ["created_at", "updated_at"]


class ClaimCareTeamProviderReferenceSerializer(BaseReferenceModelSerializer):
    """Claim Care team provider reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimCareTeamProviderReference
        exclude = ["created_at", "updated_at"]


class ClaimProviderReferenceSerializer(BaseReferenceModelSerializer):
    """Claim provider reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimProviderReference
        exclude = ["created_at", "updated_at"]


class ClaimItemRequestReferenceSerializer(BaseReferenceModelSerializer):
    """Claim item request reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimItemRequestReference
        exclude = ["created_at", "updated_at"]


class ClaimItemDetailSubDetailSerializer(WritableNestedModelSerializer):
    """Claim item detail sub detail serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimItemDetailSubDetail
        exclude = ["created_at", "updated_at"]


class ClaimPayeeSerializer(WritableNestedModelSerializer):
    """Claim payee serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    party = ClaimPayeePartyReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimPayee
        exclude = ["created_at", "updated_at"]


class ClaimItemDetailSerializer(WritableNestedModelSerializer):
    """Claim item detail serializer."""

    sequence = serializers.IntegerField(required=False)
    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)
    sub_detail = ClaimItemDetailSubDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimItemDetail
        exclude = ["created_at", "updated_at"]


class ClaimItemBodySiteSerializer(WritableNestedModelSerializer):
    """Claim item body site serializer."""

    site = BodyStructureCodeableReferenceSerializer(many=True, required=False)
    sub_site = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimItemBodySIte
        exclude = ["created_at", "updated_at"]


class ClaimItemSerializer(WritableNestedModelSerializer):
    """Claim item serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    request = ClaimItemRequestReferenceSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    serviced_date = serializers.DateField(required=False)
    serviced_period = PeriodSerializer(many=False, required=False)
    location_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    location_address = AddressSerializer(many=False, required=False)
    location_reference = LocationReferenceSerializer(many=False, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)
    body_site = ClaimItemBodySiteSerializer(many=True, required=False)
    encounter = EncounterReferenceSerializer(many=True, required=False)
    detail = ClaimItemDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimItem
        exclude = ["created_at", "updated_at"]


class ClaimEventSerializer(WritableNestedModelSerializer):
    """Claim event serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    when_period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimEvent
        exclude = ["created_at", "updated_at"]


class ClaimRelatedSerializer(WritableNestedModelSerializer):
    """Claim related serializer."""

    claim = ClaimReferenceSerializer(many=False, required=False)
    relationship = CodeableConceptSerializer(many=False, required=False)
    reference = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimRelated
        exclude = ["created_at", "updated_at"]


class ClaimAccidentSerializer(WritableNestedModelSerializer):
    """Claim accident serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    location_address = AddressSerializer(many=False, required=False)
    location_reference = LocationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimAccident
        exclude = ["created_at", "updated_at"]


class ClaimInsuranceSerializer(WritableNestedModelSerializer):
    """Claim insurance serializer."""

    identifier = IdentifierSerializer(many=False, required=False)
    coverage = CoverageReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimInsurance
        exclude = ["created_at", "updated_at"]


class ClaimProcedureSerializer(WritableNestedModelSerializer):
    """Claim procedure serializer."""

    type = CodeableConceptSerializer(many=True, required=False)
    procedure_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    procedure_reference = ProcedureReferenceSerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimProcedure
        exclude = ["created_at", "updated_at"]


class ClaimDiagnosisSerializer(WritableNestedModelSerializer):
    """Claim diagnosis serializer."""

    diagnosis_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    diagnosis_reference = ReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    on_admission = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimDiagnosis
        exclude = ["created_at", "updated_at"]


class ClaimCareTeamSerializer(WritableNestedModelSerializer):
    """Claim care team serializer."""

    provider = ClaimCareTeamProviderReferenceSerializer(many=False, required=False)
    role = CodeableConceptSerializer(many=False, required=False)
    specialty = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimCareTeam
        exclude = ["created_at", "updated_at"]


class ClaimSupportingInfoSerializer(WritableNestedModelSerializer):
    """Claim supporting info serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    timing_period = PeriodSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_attachment = AttachmentSerializer(many=False, required=False)
    value_reference = ReferenceSerializer(many=False, required=False)
    value_identifier = IdentifierSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimSupportingInfo
        exclude = ["created_at", "updated_at"]


class ClaimEntererReferenceSerializer(BaseReferenceModelSerializer):
    """Claim enterer reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimEntererReference
        exclude = ["created_at", "updated_at"]


class ClaimPrescriptionReferenceSerializer(BaseReferenceModelSerializer):
    """Claim prescription reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimPrescriptionReference
        exclude = ["created_at", "updated_at"]


class ClaimOriginalPrescriptionReferenceSerializer(BaseReferenceModelSerializer):
    """Claim original prescription reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimOriginalPrescriptionReference
        exclude = ["created_at", "updated_at"]


class ClaimSerializer(BaseWritableNestedModelSerializer):
    """Claim serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    trace_number = IdentifierSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    sub_type = CodeableConceptSerializer(many=False, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    billable_period = PeriodSerializer(many=False, required=False)
    enterer = ClaimEntererReferenceSerializer(many=False, required=False)
    insurer = OrganizationReferenceSerializer(many=False, required=False)
    provider = ClaimProviderReferenceSerializer(many=False, required=False)
    priority = CodeableConceptSerializer(many=False, required=False)
    funds_reserve = CodeableConceptSerializer(many=False, required=False)
    related = ClaimRelatedSerializer(many=True, required=False)
    prescription = ClaimPrescriptionReferenceSerializer(many=False, required=False)
    original_prescription = ClaimOriginalPrescriptionReferenceSerializer(
        many=False, required=False
    )
    payee = ClaimPayeeSerializer(many=False, required=False)
    referral = ServiceRequestReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=True, required=False)
    facility = LocationOrganizationReferenceSerializer(many=False, required=False)
    diagnosis_related_group = CodeableConceptSerializer(many=False, required=False)
    event = ClaimEventSerializer(many=True, required=False)
    care_team = ClaimCareTeamSerializer(many=True, required=False)
    supporting_info = ClaimSupportingInfoSerializer(many=True, required=False)
    diagnosis = ClaimDiagnosisSerializer(many=True, required=False)
    procedure = ClaimProcedureSerializer(many=True, required=False)
    insurance = ClaimInsuranceSerializer(many=True, required=False)
    accident = ClaimAccidentSerializer(many=False, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    item = ClaimItemSerializer(many=True, required=False)
    total = MoneySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Claim
        exclude = ["created_at", "updated_at"]



"""payment reconciliations views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.paymentreconciliations.models import PaymentReconciliation
from dfhir.paymentreconciliations.serializers import (
    PaymentReconciliationSerializer,
)


class PaymentReconciliationListView(APIView):
    """Payment reconciliation list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PaymentReconciliationSerializer(many=True)})
    def get(self, request):
        """Get all payment reconciliations."""
        payment_reconciliations = PaymentReconciliation.objects.all()
        serializer = PaymentReconciliationSerializer(payment_reconciliations, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=PaymentReconciliationSerializer,
        responses={201: PaymentReconciliationSerializer},
    )
    def post(self, request):
        """Create a payment reconciliation."""
        serializer = PaymentReconciliationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class PaymentReconciliationDetailView(APIView):
    """Payment reconciliation detail view."""

    def get_object(self, pk):
        """Get payment reconciliation object."""
        try:
            return PaymentReconciliation.objects.get(pk=pk)
        except PaymentReconciliation.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: PaymentReconciliationSerializer})
    def get(self, request, pk):
        """Get a payment reconciliation."""
        payment_reconciliation = self.get_object(pk)
        serializer = PaymentReconciliationSerializer(payment_reconciliation)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=PaymentReconciliationSerializer,
        responses={200: PaymentReconciliationSerializer},
    )
    def patch(self, request, pk):
        """Update a payment reconciliation."""
        payment_reconciliation = self.get_object(pk)
        serializer = PaymentReconciliationSerializer(
            payment_reconciliation, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a payment reconciliation."""
        payment_reconciliation = self.get_object(pk)
        payment_reconciliation.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)
